# Animated image JS library
## Example
    $(function () {

      var options = {
        'width': 813,
        'height': 614,
        'id': 'some-id',
        'animationSets': [
          new Animation([
            {
              'imagesrc': 'images/chart-bg.jpg',
              'position': {
                x: 0,
                y: 0
              },
              'width': 813,
              'height': 614,
            }, {
              'imagesrc': 'images/gray-line-1.png',
              'position': {
                x: 164,
                y: 94
              },
              'width': 221,
              'height': 212,
              'type': 'reveal',
              'duration': 500,
            }, {
              'imagesrc': 'images/orange-line-1.png',
              'position': {
                x: 164,
                y: 90
              },
              'width': 221,
              'height': 125,
              'type': 'reveal',
              'duration': 2000,
              'delay': 1000
            }
          ]),
          new Animation([
            {
              'imagesrc': 'images/orange-explanation.png',
              'position': {
                x: 634,
                y: 150
              },
              'width': 179,
              'height': 100,
              'type': 'fadeIn',
              'duration': 500,
              'id': 'fadeout-2',
              'delay': 100
            }, {
              'imagesrc': 'images/gray-explanation.png',
              'position': {
                x: 170,
                y: 321
              },
              'width': 179,
              'height': 100,
              'type': 'fadeIn',
              'duration': 500,
              'id': 'fadeout-1',
            }
          ]),
          new Animation([
            {
              'imagesrc': 'images/2-lines.png',
              'position': {
                x: 385,
                y: 214
              },
              'width': 244,
              'height': 153,
              'type': 'reveal',
              'duration': 1000,
            }, {
              'type': 'fadeOut',
              'duration': 500,
              'id': 'fadeout-1',
            }, {
              'id': 'fadeout-2',
              'type': 'fadeOut',
              'duration': 500,
              'delay': 250
            }
          ], true);
        ]
      };

      var a = new AnimationLoader('body', options);

      a.init();
    });
## How to use
### AnimationLoader Function
Pass a CSS selector and an object to the `AnimationLoader` function. The `init` method starts the animation.

The container generated by this library will be appended inside the selector that was passed in.

The object should have `width`, `height`, `animationSets` and `id` properties. Width and height determine the container size.

- `width` integer - width of the container
- `height` integer - height of the container
- `id` string - sets the id attribute of the container to this value
- `animationSets` array of `Animation` objects

### Animation Function

Takes an array of objects and an optional boolean `Animation([obj1, obj2], true)`.

The boolean is a flag to pause and wait for a click(or tap) when it is that animation set's turn. It defaults to `false`.

Each object in the array that is passed to the Animation function must have certain properties:

  - `type` can be set to `reveal`, `fadeIn`, or `fadeOut`. If the type is not set, the image will show immediately when it is its turn in the queue.
  - `imagesrc` is the path to the image. This should be the path from the HTML file that this library is being used on.
  - `position` is an object that has an `x` and `y` property. This is to set the absolute positioning within the containing element.
  - `duration` is the length in time in milliseconds for the animation to complete. Default is 400ms
  - `delay` optional, delays the start of the animation. Takes a number in milliseconds. Default is 0
  - `id` optional. Defaults to a randomly generated guid-like string. This is intended to be used with `fadeOut` animation types - if you want to fade out an image that has been previously animated. For example, if you want to fade something in and then out at a certain time, you pass the id to the Animation function when you set the fade in options and then you pass the same id when you set the fade out options. The example settings below will fade in the image and then wait for a click to fade it out.

          new Animation([
            {
              'imagesrc': 'images/gray-explanation.png',
              'position': {
                x: 170,
                y: 321
              },
              'width': 179,
              'height': 100,
              'type': 'fadeIn',
              'duration': 500,
              'id': 'fadeout-1',
            }
          ]),
          new Animation([
            {
              'type': 'fadeOut',
              'duration': 500,
              'id': 'fadeout-1',
            }
          ], true);

Note that you don't have to pass an imagesrc, position, width, or height to the Animation function when you do the fade out.

### Some things to understand about this library

The elements in the `animationSets` array that are passed to the AnimationLoader function get animated in order. You should group animations that you need to play simultaneously in the same array that is passed to a single Animation function. You can think of each call to an Animation function in the example at the top as a point in time when related animations will start, however a delay to any single animation in a set can be added. But, a set will not start until the previous set of animations has finished.

### Dependencies

jQuery 1.7+

### Changelog

#### 1.2 - 11-17-2017
Added
- have multiple animations play simultaneously
- added fade out

#### 1.1 - 10-18-2017
Added
- added delay feature to each animation

#### 1.0.3 - 10-18-2017
Added
- added click feature option to each animation instead of having it all or nothing

## To Do
Remove jQuery